# Copyright 2016 Canonical Ltd.
# Copyright 2017-18 Telefonica
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
##

from charmhelpers.core.hookenv import (
    action_get,
    action_fail,
    action_set,
    status_set,
    config,
    log
)
from charms.reactive import (
    clear_flag,
    set_flag,
    when,
    when_not,
)
import charms.sshproxy
import sys, traceback
from jinja2 import Environment, FileSystemLoader


@when('sshproxy.configured')
@when_not('mavenir-vhss-conf.installed')
def install_proxy_charm():
    """Post-install actions.
    This function will run when two conditions are met:
    1. The 'sshproxy.configured' state is set
    2. The 'mavenir-vhss-conf.installed' state is not set
    This ensures that the workload status is set to active only when the SSH
    proxy is properly configured.
    """
    set_flag('mavenir-vhss-conf.installed')
    status_set('active', 'Ready!')

@when('actions.touch')
def touch():
    err = ''
    try:
        filename = action_get('filename')
        cmd = ['touch {}'.format(filename)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.touch')

@when('actions.day1-configuration')
def day1Configuration():
    err = ''
    try:
        # The code below is only a placeholder that has to be modified
        # It retrieves the params, and creates a command that will be
        # executed via ssh

        # Step 1. Initialize Database
        cmd = "cd /opt/VCM/etc/scripts/udb-scripts/; /opt/VCM/etc/scripts/udb-scripts/start-hss_data-single_udb_auto.sh"
        result, err = charms.sshproxy._run(cmd)
        
        log("Database initialize command executed.")
        
        # Step 2. Get params
        hss_origin_host_name = action_get('hss_origin_host_name')
        hss_host_realm_name = action_get('hss_host_realm_name')
        hss_ip_addr = action_get('hss_ip_addr')
        hss_ip_addr_mask = action_get('hss_ip_addr_mask')
        hss_interface = action_get('hss_interface')
        trans_key_name = action_get('trans_key_name')
        trans_key_value = action_get('trans_key_value')

        #'command {}'.format(ip_pool_name)

        # Step 3. Generate initial_config.cfg from token_files/sample_day1_configuration.cfg
        #         properly substituting the params. Suggestion to use jinja2
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_day1_configuration.cfg')
        output_from_parsed_template = template.render (hss_origin_host_name = hss_origin_host_name,
            hss_host_realm_name = hss_host_realm_name,
            hss_ip_addr = hss_ip_addr,
            hss_ip_addr_mask = hss_ip_addr_mask,
            hss_interface = hss_interface,
            trans_key_name = trans_key_name,
            trans_key_value = trans_key_value)

        #print output_from_parsed_template

        # save the results in a temporary file
        temp_file="commands_day1_configuration.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 4. Copy commands_day1_configuration.cfg to VEM:/opt/VCM/config/
        # doScp(user,password,host,remotepath,localpath,files)
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        # Delete local temp_file (the file itself, not the variable)
        # Step 5. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("day1-configuration: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.day1-configuration')

@when('actions.add-epc')
def addEPC():
    err = ''
    try:
        # Step 1. Get params
        tef_vepc_name = action_get('tef_vepc_name')
        s6a_mme_host_name = action_get('s6a_mme_host_name')
        s6a_mme_realm_name = action_get('s6a_mme_realm_name')
        s6a_bind_ip_address = action_get('s6a_bind_ip_address')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_add_epc.cfg')
        output_from_parsed_template = template.render (tef_vepc_name = tef_vepc_name,
            s6a_mme_host_name = s6a_mme_host_name,
            s6a_mme_realm_name = s6a_mme_realm_name,
            s6a_bind_ip_address = s6a_bind_ip_address)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_add_epc.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("add-epc: DONE")
    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.add-epc')

@when('actions.new-imsi')
def newImsi():
    err = ''
    try:
        # Step 1. Get params
        # new imsi
        trans_key_name = action_get('trans_key_name')
        encrypted_opc_key = action_get('encrypted_opc_key')
        encrypted_ki_key = action_get('encrypted_ki_key')
        imsi_provisioning = action_get('imsi_provisioning')
        subs_template_name = action_get('subs_template_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_imsi.cfg')
        output_from_parsed_template = template.render (trans_key_name = trans_key_name,
            encrypted_opc_key = encrypted_opc_key,
            encrypted_ki_key = encrypted_ki_key,
            imsi_provisioning = imsi_provisioning,
            subs_template_name = subs_template_name)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_imsi.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-imsi: DONE")
    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-imsi')

@when('actions.new-apn')
def newAPN():
    err = ''
    try:
        # Step 1. Get params
        # new APN
        apn_name = action_get('apn_name')
        uplink_apn_ambr = action_get('uplink_apn_ambr')
        downlink_apn_ambr = action_get('downlink_apn_ambr')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_apn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name,
            uplink_apn_ambr = uplink_apn_ambr,
            downlink_apn_ambr = downlink_apn_ambr)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_apn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-apn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-apn')

@when('actions.new-qos-profile')
def newQoSProfile():
    err = ''
    try:
        # Step 1. Get params
        # new qos profile
        eps_qos_profile = action_get('eps_qos_profile')
        eps_arp_value = action_get('eps_arp_value')
        eps_qci_value = action_get('eps_qci_value')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_qos_profile.cfg')
        output_from_parsed_template = template.render (eps_qos_profile = eps_qos_profile,
            eps_arp_value = eps_arp_value,
            eps_qci_value = eps_qci_value)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_qos_profile.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-qos-profile: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-qos-profile')

@when('actions.new-service-profile')
def newServiceProfile():
    err = ''
    try:
        # Step 1. Get params
        # new service profile
        apn_name = action_get('apn_name')
        eps_qos_profile = action_get('eps_qos_profile')
        eps_service_prof = action_get('eps_service_prof')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_service_profile.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name,
            eps_qos_profile = eps_qos_profile,
            eps_service_prof = eps_service_prof)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_service_profile.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-service-profile: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-service-profile')

@when('actions.new-subscriber-template')
def newSubscriberTemplate():
    err = ''
    try:
        # Step 1. Get params
        # new service profile
        subs_template_name = action_get('subs_template_name')
        uplink_ue_ambr = action_get('uplink_ue_ambr')
        downlink_ue_ambr = action_get('downlink_ue_ambr')
        eps_service_prof = action_get('eps_service_prof')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_subscriber_template.cfg')
        output_from_parsed_template = template.render (subs_template_name = subs_template_name,
            uplink_ue_ambr = uplink_ue_ambr,
            downlink_ue_ambr = downlink_ue_ambr,
            eps_service_prof = eps_service_prof)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_subscriber_template.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-subscriber-template: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-subscriber-template')

@when('actions.remove-epc')
def removeEPC():
    err = ''
    try:
        # Step 1. Get params
        tef_vepc_name = action_get('tef_vepc_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_epc.cfg')
        output_from_parsed_template = template.render (tef_vepc_name = tef_vepc_name)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_epc.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-epc: DONE")
    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-epc')

@when('actions.remove-imsi')
def removeImsi():
    err = ''
    try:
        # Step 1. Get params
        # remove imsi
        imsi_provisioning = action_get('imsi_provisioning')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_imsi.cfg')
        output_from_parsed_template = template.render (imsi_provisioning = imsi_provisioning)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_imsi.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-imsi: DONE")
    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-imsi')

@when('actions.remove-apn')
def removeAPN():
    err = ''
    try:
        # Step 1. Get params
        # remove APN
        apn_name = action_get('apn_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_apn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_apn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-apn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-apn')

@when('actions.remove-qos-profile')
def removeQoSProfile():
    err = ''
    try:
        # Step 1. Get params
        # remove qos profile
        eps_qos_profile = action_get('eps_qos_profile')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_qos_profile.cfg')
        output_from_parsed_template = template.render (eps_qos_profile = eps_qos_profile)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_qos_profile.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-qos-profile: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-qos-profile')

@when('actions.remove-service-profile')
def removeServiceProfile():
    err = ''
    try:
        # Step 1. Get params
        # remove service profile
        eps_service_prof = action_get('eps_service_prof')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_service_profile.cfg')
        output_from_parsed_template = template.render (eps_service_prof = eps_service_prof)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_service_profile.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-service-profile: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-service-profile')

@when('actions.remove-subscriber-template')
def removeSubscriberTemplate():
    err = ''
    try:
        # Step 1. Get params
        # remove service profile
        subs_template_name = action_get('subs_template_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_subscriber_template.cfg')
        output_from_parsed_template = template.render (subs_template_name = subs_template_name)

        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_subscriber_template.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vhss " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-subscriber-template: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-subscriber-template')

